#!python
# coding=utf-8
import os
import unittest
import tempfile

from numpy import testing as npt

from pocean.cf import CFDataset
from pocean.dataset import EnhancedDataset
from pocean.meta import MetaInterface, ncpyattributes

import logging
from pocean import logger as L
L.level = logging.INFO
L.handlers = [logging.StreamHandler()]


class TestJsonDataset(unittest.TestCase):

    def setUp(self):
        self.maxDiff = 9999
        self.hdl, self.ncdf = tempfile.mkstemp(prefix='pocean_test_')

    def tearDown(self):
        os.close(self.hdl)
        os.remove(self.ncdf)

    def test_lvl0_apply(self):
        jsf = os.path.join(os.path.dirname(__file__), "resources/coamps_lvl0.json")
        mi = MetaInterface.from_jsonfile(jsf)

        with EnhancedDataset(self.ncdf, 'w') as ncd:
            ncd.apply_meta(mi)

            assert { k: v.size for k, v in ncd.dimensions.items() } == mi['dimensions']

            fileglobatts = mi['attributes']
            newglobatts = {}
            for nk in ncd.ncattrs():
                newglobatts[nk] = ncd.getncattr(nk)

            self.assertDictEqual(
                fileglobatts,
                newglobatts
            )

            for k, v in ncd.variables.items():

                filevaratts = mi['variables'][k]['attributes']
                newvaratts = ncpyattributes(dict(v.__dict__), verbose=False)

                # _FillValue gets added even if it wasn't in the original attributes
                if '_FillValue' in newvaratts:
                    del newvaratts['_FillValue']

                if 'missing_value' in filevaratts:
                    del filevaratts['missing_value']

                self.assertDictEqual(
                    filevaratts,
                    newvaratts
                )

    def test_lvl2_apply(self):
        jsf = os.path.join(os.path.dirname(__file__), "resources/coamps_lvl2.json")
        mi = MetaInterface.from_jsonfile(jsf)

        with EnhancedDataset(self.ncdf, 'w') as ncd:
            ncd.apply_meta(mi)

            assert { k: v.size for k, v in ncd.dimensions.items() } == mi['dimensions']

            fileglobatts = { k: v['data'] for k, v in mi['attributes'].items() }
            newglobatts = {}
            for nk in ncd.ncattrs():
                newglobatts[nk] = ncd.getncattr(nk)

            self.assertDictEqual(
                fileglobatts,
                newglobatts
            )

            for k, v in ncd.variables.items():

                filevaratts = { k: v['data'] for k, v in mi['variables'][k]['attributes'].items() }
                newvaratts = ncpyattributes(dict(v.__dict__), verbose=False)

                # _FillValue gets added even if it wasn't in the original attributes
                if '_FillValue' in newvaratts:
                    del newvaratts['_FillValue']

                if 'missing_value' in filevaratts:
                    del filevaratts['missing_value']

                self.assertDictEqual(
                    filevaratts,
                    newvaratts
                )

    def test_input_output(self):
        ncfile = os.path.join(os.path.dirname(__file__), "resources/coamps.nc")

        with EnhancedDataset(ncfile, 'r') as original_ncd:
            mi = original_ncd.meta()

            with EnhancedDataset(self.ncdf, 'w') as ncd:
                ncd.apply_meta(mi)

                self.assertDictEqual(
                    ncpyattributes(dict(original_ncd.__dict__)),
                    ncpyattributes(dict(ncd.__dict__))
                )

                for k, v in original_ncd.variables.items():

                    oldatts = ncpyattributes(dict(v.__dict__))
                    newatts = ncpyattributes(dict(ncd.variables[k].__dict__))

                    # _FillValue gets added even if it wasn't in the original attributes
                    if '_FillValue' in newatts:
                        del newatts['_FillValue']

                    if 'missing_value' in oldatts:
                        del oldatts['missing_value']

                    self.assertDictEqual(
                        oldatts,
                        newatts
                    )

    def test_serialize_and_reload_data(self):
        ncfile = os.path.join(os.path.dirname(__file__), "resources/qc-month.nc")

        with CFDataset(ncfile) as cfncd:

            # Data from netCDF variable
            ncdata = cfncd.variables['data1'][:]

            # Not filled
            meta = cfncd.json(return_data=True, fill_data=False)
            jsdata = meta['variables']['data1']['data']
            npt.assert_array_equal(ncdata, jsdata)
            fhandle1, fname1 = tempfile.mkstemp()
            with CFDataset(fname1, 'w') as newcf:
                newcf.apply_json(meta)
            with CFDataset(fname1, 'r') as rcf:
                newncdata = rcf.variables['data1'][:]
                npt.assert_array_equal(ncdata, newncdata)
            os.close(fhandle1)
            os.remove(fname1)

            # Filled
            meta = cfncd.json(return_data=True, fill_data=True)
            jsdata = meta['variables']['data1']['data']
            npt.assert_array_equal(ncdata, jsdata)
            fhandle2, fname2 = tempfile.mkstemp()
            with CFDataset(fname2, 'w') as newcf:
                newcf.apply_json(meta)

            with CFDataset(fname2, 'r') as rcf:
                newncdata = rcf.variables['data1'][:]
                npt.assert_array_equal(ncdata, newncdata)

            os.close(fhandle2)
            os.remove(fname2)



#!python
# coding=utf-8
import os
import unittest

from pocean.cf import CFDataset
from pocean.dsg import OrthogonalMultidimensionalTimeseries as omt

import logging
from pocean import logger as L
L.level = logging.INFO
L.handlers = [logging.StreamHandler()]


class TestCFDatasetLoad(unittest.TestCase):

    def test_load_url(self):
        ncd = CFDataset.load('http://geoport.whoi.edu/thredds/dodsC/usgs/data2/emontgomery/stellwagen/CF-1.6/ARGO_MERCHANT/1211-AA.cdf')
        assert omt.is_mine(ncd) is True
        ncd.close()

    def test_load_strict(self):
        ncfile = os.path.join(os.path.dirname(__file__), 'dsg', 'profile', 'resources', 'om-single.nc')

        ncd = CFDataset.load(ncfile)
        assert omt.is_mine(ncd) is False
        with self.assertRaises(BaseException):
            omt.is_mine(ncd, strict=True)
        ncd.close()



#!python
# coding=utf-8
import os
import shutil
import unittest
import tempfile

import numpy as np
import netCDF4 as nc4

from pocean.dataset import EnhancedDataset
from pocean.utils import generic_masked, get_default_axes, normalize_array

import logging
from pocean import logger
logger.level = logging.INFO
logger.handlers = [logging.StreamHandler()]


class TestUtils(unittest.TestCase):

    def setUp(self):
        self.input_file = os.path.join(os.path.dirname(__file__), "resources/coamps.nc")

    def test_get_default_axes(self):
        assert get_default_axes() == (
            'trajectory',
            'station',
            'profile',
            'obs',
            't',
            'x',
            'y',
            'z',
        )

        new_defaults = {
            'trajectory': 'a',
            'station':    'b',
            'profile':    'c',
            'sample':     'h',
            't':          'd',
            'x':          'e',
            'y':          'f',
            'z':          'g',
        }
        assert get_default_axes(new_defaults) == (
            'a',
            'b',
            'c',
            'h',
            'd',
            'e',
            'f',
            'g',
        )

        new_defaults = {
            'trajectory': 'a',
            'station':    'b',
            'profile':    'c'
        }
        assert get_default_axes(new_defaults) == (
            'a',
            'b',
            'c',
            'obs',
            't',
            'x',
            'y',
            'z',
        )

        # Time is not a valid axis key
        bad_defaults = {
            'time': 'a'
        }
        with self.assertRaises(TypeError):
            get_default_axes(bad_defaults)

        # Can't have duplicate values
        bad_defaults = {
            'x': 'a',
            'y': 'a'
        }
        with self.assertRaises(ValueError):
            get_default_axes(bad_defaults)

        # but you can with the sample dimension
        bad_defaults = {
            't': 'time',
            'sample': 'time'
        }
        assert get_default_axes(bad_defaults) == (
            'trajectory',
            'station',
            'profile',
            'time',
            'time',
            'x',
            'y',
            'z',
        )

    def test_single_attr_filter(self):
        nc = EnhancedDataset(self.input_file)
        grid_spacing_vars = nc.filter_by_attrs(grid_spacing='4.0 km')

        x = nc.variables.get('x')
        y = nc.variables.get('y')

        self.assertEqual(len(grid_spacing_vars), 2)
        assert x in grid_spacing_vars
        assert y in grid_spacing_vars

    def test_multiple_attr_filter(self):
        nc = EnhancedDataset(self.input_file)
        grid_spacing_vars = nc.filter_by_attrs(grid_spacing='4.0 km', standard_name='projection_y_coordinate')

        y = nc.variables.get('y')

        self.assertEqual(len(grid_spacing_vars), 1)
        assert y in grid_spacing_vars

    def test_generic_masked_bad_min_max_value(self):

        fid, tpath = tempfile.mkstemp(suffix='.nc', prefix='pocean-test')
        shutil.copy2(self.input_file, tpath)

        with EnhancedDataset(tpath, 'a') as ncd:
            v = ncd.variables['v_component_wind_true_direction_all_geometries']
            v.valid_min = 0.1
            v.valid_max = 0.1
            r = generic_masked(v[:], attrs=ncd.vatts(v.name))
            rflat = r.flatten()
            assert rflat[~rflat.mask].size == 0

            # Create a byte variable with a float valid_min and valid_max
            # to make sure it doesn't error
            b = ncd.createVariable('imabyte', 'b')
            b.valid_min = 0
            b.valid_max = 600  # this is over a byte and thus invalid
            b[:] = 3
            r = generic_masked(b[:], attrs=ncd.vatts(b.name))
            assert np.all(r.mask == False)  # noqa

            b.valid_min = 0
            b.valid_max = 2
            r = generic_masked(b[:], attrs=ncd.vatts(b.name))
            assert np.all(r.mask == True)  # noqa

            c = ncd.createVariable('imanotherbyte', 'f4')
            c.setncattr('valid_min', b'0')
            c.setncattr('valid_max', b'9')
            c[:] = 3
            r = generic_masked(c[:], attrs=ncd.vatts(c.name))
            assert np.all(r.mask == False)  # noqa

            c = ncd.createVariable('imarange', 'f4')
            c.valid_range = [0.0, 2.0]
            c[:] = 3.0
            r = generic_masked(c[:], attrs=ncd.vatts(c.name))
            assert np.all(r.mask == True)  # noqa

            c.valid_range = [0.0, 2.0]
            c[:] = 1.0
            r = generic_masked(c[:], attrs=ncd.vatts(c.name))
            assert np.all(r.mask == False)  # noqa

        os.close(fid)
        if os.path.exists(tpath):
            os.remove(tpath)


class TestNetcdfUtils(unittest.TestCase):

    def test_cf_safe_name(self):
        from pocean.cf import cf_safe_name
        self.assertEqual('foo', cf_safe_name('foo'))
        self.assertEqual('v_1foo', cf_safe_name('1foo'))
        self.assertEqual('v_1foo_99', cf_safe_name('1foo-99'))
        self.assertEqual('foo_99', cf_safe_name('foo-99'))
        self.assertEqual('foo_99_', cf_safe_name('foo(99)'))
        self.assertEqual('v__foo_99_', cf_safe_name('_foo(99)'))


class TestNormalizeArray(unittest.TestCase):

    def setUp(self):
        self.fh, self.fp = tempfile.mkstemp(suffix='.nc', prefix='pocean_testing_')

    def tearDown(self):
        os.close(self.fh)
        if os.path.exists(self.fp):
            os.remove(self.fp)

    def test_normalization_of_string_arrays_netcdf4(self):
        thestr = 'bosadfsdfkljskfusdiofu987987987om'

        with nc4.Dataset(self.fp, 'w', format="NETCDF4") as ncd:

            dimsize = len(thestr)
            ncd.createDimension('n', dimsize)

            # Single str (no dimension)
            ncd.createVariable('single_str', str)
            ncd.createVariable('single_unicode_', np.unicode_)
            ncd.createVariable('single_U', '<U1')
            ncd.createVariable('single_S', 'S1', ('n',))

            for k, v in ncd.variables.items():
                if k.startswith('single_'):
                    if v.dimensions:
                        v[:] = nc4.stringtoarr(thestr, dimsize)
                    else:
                        v[0] = thestr

            # Array of str
            ncd.createVariable('many_str', str, ('n',))
            ncd.createVariable('many_unicode_', np.unicode_, ('n',))
            ncd.createVariable('many_U', '<U1', ('n',))
            ncd.createVariable('many_S', 'S1', ('n', 'n',))

            for k, v in ncd.variables.items():
                if k.startswith('many_'):
                    if len(v.dimensions) > 1:
                        v[:, :] = np.tile(nc4.stringtoarr(thestr, dimsize), dimsize).reshape(v.shape)
                    else:
                        v[:] = np.tile(thestr, dimsize).reshape(v.shape)

        with nc4.Dataset(self.fp) as ncd:
            assert normalize_array(ncd.variables['single_str']) == thestr
            assert normalize_array(ncd.variables['single_unicode_']) == thestr
            assert normalize_array(ncd.variables['single_U']) == thestr
            assert normalize_array(ncd.variables['single_S']) == thestr

            assert np.all(normalize_array(ncd.variables['many_str']) == [thestr] * len(thestr))
            assert np.all(normalize_array(ncd.variables['many_unicode_']) == [thestr] * len(thestr))
            assert np.all(normalize_array(ncd.variables['many_U']) == [thestr] * len(thestr))
            assert np.all(normalize_array(ncd.variables['many_S']) == [thestr] * len(thestr))

    def test_normalization_of_string_arrays_netcdf3(self):
        thestr = 'boodsfasfasdfm'

        with nc4.Dataset(self.fp, 'w', format="NETCDF3_CLASSIC") as ncd:

            dimsize = len(thestr)
            ncd.createDimension('n', dimsize)

            # Single str (no dimension)
            ncd.createVariable('single_S', 'S1', ('n',))

            for k, v in ncd.variables.items():
                if k.startswith('single_'):
                    v[:] = nc4.stringtoarr(thestr, dimsize)

            # Array of strq
            ncd.createVariable('many_S', 'S1', ('n', 'n',))

            for k, v in ncd.variables.items():
                if k.startswith('many_'):
                    v[:, :] = np.tile(nc4.stringtoarr(thestr, dimsize), dimsize).reshape(v.shape)

        with nc4.Dataset(self.fp) as ncd:
            assert normalize_array(ncd.variables['single_S']) == thestr
            assert np.all(normalize_array(ncd.variables['many_S']) == [thestr] * dimsize)



#!python
# coding=utf-8
from pocean.cf import CFDataset


class IncompleteMultidimensionalTimeseriesProfile(CFDataset):

    @classmethod
    def is_mine(cls, dsg, strict=False):
        try:
            assert dsg.featureType.lower() == 'timeseriesprofile'
            assert len(dsg.t_axes()) >= 1
            assert len(dsg.x_axes()) >= 1
            assert len(dsg.y_axes()) >= 1
            assert len(dsg.z_axes()) >= 1

            zvar = dsg.z_axes()[0]
            assert len(zvar.dimensions) > 1

            # Not ragged
            o_index_vars = dsg.filter_by_attrs(
                sample_dimension=lambda x: x is not None
            )
            assert len(o_index_vars) == 0

            r_index_vars = dsg.filter_by_attrs(
                instance_dimension=lambda x: x is not None
            )
            assert len(r_index_vars) == 0

        except BaseException:
            if strict is True:
                raise
            return False

        return True

    def from_dataframe(cls, df, output, **kwargs):
        raise NotImplementedError

    def calculated_metadata(self, df=None, geometries=True, clean_cols=True, clean_rows=True):
        # if df is None:
        #     df = self.to_dataframe(clean_cols=clean_cols, clean_rows=clean_rows)
        raise NotImplementedError

    def to_dataframe(self):
        raise NotImplementedError



# -*- coding: utf-8 -*-
from os.path import join as jn
from os.path import dirname as dn

import pytest

from pocean.cf import CFDataset
from pocean.utils import all_subclasses
from pocean.dsg import *

import logging
from pocean import logger
logger.level = logging.INFO
logger.handlers = [logging.StreamHandler()]


@pytest.mark.parametrize("klass,fp", [
    (OrthogonalMultidimensionalProfile,           jn(dn(__file__), 'profile', 'resources', 'om-single.nc')),
    (OrthogonalMultidimensionalProfile,           jn(dn(__file__), 'profile', 'resources', 'om-multiple.nc')),
    (OrthogonalMultidimensionalProfile,           jn(dn(__file__), 'profile', 'resources', 'om-1dy11.nc')),
    (IncompleteMultidimensionalProfile,           jn(dn(__file__), 'profile', 'resources', 'im-multiple.nc')),
    (IncompleteMultidimensionalTrajectory,        jn(dn(__file__), 'trajectory', 'resources', 'im-single.nc')),
    (IncompleteMultidimensionalTrajectory,        jn(dn(__file__), 'trajectory', 'resources', 'im-multiple.nc')),
    (IncompleteMultidimensionalTrajectory,        jn(dn(__file__), 'trajectory', 'resources', 'im-multiple-nonstring.nc')),
    (IncompleteMultidimensionalTrajectory,        jn(dn(__file__), 'trajectory', 'resources', 'wave-glider-int-attrs.nc')),
    (ContiguousRaggedTrajectory,                  jn(dn(__file__), 'trajectory', 'resources', 'cr-multiple.nc')),
    (ContiguousRaggedTrajectory,                  jn(dn(__file__), 'trajectory', 'resources', 'cr-oot-A.nc')),
    (ContiguousRaggedTrajectory,                  jn(dn(__file__), 'trajectory', 'resources', 'cr-oot-B.nc')),
    (ContiguousRaggedTrajectoryProfile,           jn(dn(__file__), 'trajectoryProfile', 'resources', 'cr-single.nc')),
    (ContiguousRaggedTrajectoryProfile,           jn(dn(__file__), 'trajectoryProfile', 'resources', 'cr-multiple.nc')),
    (ContiguousRaggedTrajectoryProfile,           jn(dn(__file__), 'trajectoryProfile', 'resources', 'cr-missing-time.nc')),
    (IncompleteMultidimensionalTimeseries,        jn(dn(__file__), 'timeseries', 'resources', 'im-multiple.nc')),
    (OrthogonalMultidimensionalTimeseries,        jn(dn(__file__), 'timeseries', 'resources', 'om-single.nc')),
    (OrthogonalMultidimensionalTimeseries,        jn(dn(__file__), 'timeseries', 'resources', 'om-multiple.nc')),
    #(IndexedRaggedTimeseries,                     jn(dn(__file__), 'timeseries', 'resources', 'cr-multiple.nc')),
    #(ContiguousRaggedTimeseries,                  jn(dn(__file__), 'timeseries', 'resources', 'cr-multiple.nc')),
    (OrthogonalMultidimensionalTimeseriesProfile, jn(dn(__file__), 'timeseriesProfile', 'resources', 'om-multiple.nc')),
    (IncompleteMultidimensionalTimeseriesProfile, jn(dn(__file__), 'timeseriesProfile', 'resources', 'im-single.nc')),
    (IncompleteMultidimensionalTimeseriesProfile, jn(dn(__file__), 'timeseriesProfile', 'resources', 'im-multiple.nc')),
    (RaggedTimeseriesProfile,                     jn(dn(__file__), 'timeseriesProfile', 'resources', 'r-single.nc')),
    (RaggedTimeseriesProfile,                     jn(dn(__file__), 'timeseriesProfile', 'resources', 'r-multiple.nc')),
])
def test_is_mine(klass, fp):
    with CFDataset.load(fp) as dsg:
        assert dsg.__class__ == klass

    allsubs = list(all_subclasses(CFDataset))
    subs = [ s for s in allsubs if s != klass ]
    with CFDataset(fp) as dsg:
        logger.debug('\nTesting {}'.format(klass.__name__))
        assert klass.is_mine(dsg, strict=True) is True
        for s in subs:
            if hasattr(s, 'is_mine'):
                logger.debug('  * Trying {}...'.format(s.__name__))
                assert s.is_mine(dsg) is False



#!python
# coding=utf-8
import os
import unittest
from datetime import datetime, timedelta
from dateutil.parser import parse as dtparse

import pytz
import pandas as pd

from pocean.dsg import utils
from pocean.cf import CFDataset
from pocean import logger as L  # noqa


class TestDsgUtils(unittest.TestCase):

    geo = pd.DataFrame({
        'x': [-1, -2, -3, -4],
        'y': [1, 2, 3, 4]
    })

    z = pd.DataFrame({
        'z': [1, 2, 3, 4],
    })

    times = pd.DataFrame({
        't': pd.to_datetime([
            '2018-08-19 00:00:00',
            '2018-08-20 00:00:00',
            '2018-08-21 00:00:00',
            '2018-08-22 00:00:00',
            '2018-08-23 00:00:00',
            '2018-08-23 00:00:05',
        ])
    })

    avgtimes = pd.DataFrame({
        't': pd.to_datetime([
            '2018-08-19 00:00:00',
            '2018-08-20 23:00:55',
            '2018-08-21 00:00:35',
        ])
    })

    def test_get_vertical_meta(self):
        meta = utils.get_vertical_attributes(self.z)

        assert meta == {
            'variables': {
                'z': {
                    'attributes': {
                        'actual_min': 1,
                        'actual_max': 4,
                    }
                },
            },
            'attributes': {
                'geospatial_vertical_min': 1,
                'geospatial_vertical_max': 4,
                'geospatial_vertical_units': 'm',
            }
        }

    def test_get_geospatial_meta(self):
        meta = utils.get_geographic_attributes(self.geo)

        assert meta == {
            'variables': {
                'y': {
                    'attributes': {
                        'actual_min': 1,
                        'actual_max': 4,
                    }
                },
                'x': {
                    'attributes': {
                        'actual_min': -4,
                        'actual_max': -1,
                    }
                },
            },
            'attributes': {
                'geospatial_lat_min': 1.0,
                'geospatial_lat_max': 4.0,
                'geospatial_lon_min': -4.0,
                'geospatial_lon_max': -1.0,
                'geospatial_bbox': 'POLYGON ((-1 1, -1 4, -4 4, -4 1, -1 1))',
                'geospatial_bounds': 'LINESTRING (-1 1, -4 4)',
                'geospatial_bounds_crs': 'EPSG:4326',
            }
        }

    def test_get_temporal_meta_from_times_average(self):
        meta = utils.get_temporal_attributes(self.avgtimes)

        assert meta == {
            'variables': {
                't': {
                    'attributes': {
                        'actual_min': '2018-08-19T00:00:00Z',
                        'actual_max': '2018-08-21T00:00:35Z',
                    }
                }
            },
            'attributes': {
                'time_coverage_start': '2018-08-19T00:00:00Z',
                'time_coverage_end': '2018-08-21T00:00:35Z',
                'time_coverage_duration': 'P2DT0H0M35S',
                'time_coverage_resolution': 'P0DT16H0M12S',
            }
        }

    def test_get_temporal_meta_from_times(self):
        meta = utils.get_temporal_attributes(self.times)

        assert meta == {
            'variables': {
                't': {
                    'attributes': {
                        'actual_min': '2018-08-19T00:00:00Z',
                        'actual_max': '2018-08-23T00:00:05Z',
                    }
                }
            },
            'attributes': {
                'time_coverage_start': '2018-08-19T00:00:00Z',
                'time_coverage_end': '2018-08-23T00:00:05Z',
                'time_coverage_duration': 'P4DT0H0M5S',
                'time_coverage_resolution': 'P1DT0H0M0S',
            }
        }

    def test_get_creation(self):
        meta = utils.get_creation_attributes(history='DID THINGS')

        now = datetime.utcnow().replace(tzinfo=pytz.utc)

        assert (now - dtparse(meta['attributes']['date_created'])) < timedelta(minutes=1)
        assert (now - dtparse(meta['attributes']['date_issued'])) < timedelta(minutes=1)
        assert (now - dtparse(meta['attributes']['date_modified'])) < timedelta(minutes=1)
        assert 'DID THINGS' in meta['attributes']['history']

    def test_wrap_dateline(self):
        ncfile = os.path.join(os.path.dirname(os.path.dirname(__file__)), "resources/wrapping_dateline.nc")

        with CFDataset.load(ncfile) as ncd:
            axes = {
                't': 'time',
                'z': 'z',
                'x': 'lon',
                'y': 'lat',
            }
            df = ncd.to_dataframe(axes=axes)

            meta = utils.get_geographic_attributes(df, axes=axes)

            assert meta == {
                "variables": {
                    "lat": {
                        "attributes": {
                            "actual_min": 61.777,
                            "actual_max": 67.068
                        }
                    },
                    "lon": {
                        "attributes": {
                            "actual_min": -179.966,
                            "actual_max": 179.858
                        }
                    }
                },
                "attributes": {
                    "geospatial_lat_min": 61.777,
                    "geospatial_lat_max": 67.068,
                    "geospatial_lon_min": -179.966,
                    "geospatial_lon_max": 179.858,
                    "geospatial_bbox": "POLYGON ((198.669 61.777, 198.669 67.068, 174.792 67.068, 174.792 61.777, 198.669 61.777))",
                    'geospatial_bounds': "POLYGON ((174.792 61.777, 174.926 62.206, 178.838667737 64.055605136, 178.916 64.084, 179.858 64.311, 192.86 67.029, 196.86 67.068, 198.669 66.861, 187.753767857 64.334204193, 179.195 62.395, 176.169 61.862, 174.792 61.777))",
                    "geospatial_bounds_crs": "EPSG:4326"
                }
            }

    def test_wrap_small_coords(self):

        geo = pd.DataFrame({
            'x': [-1, -2],
            'y': [1, 2]
        })

        meta = utils.get_geographic_attributes(geo)

        assert meta == {
            'variables': {
                'y': {
                    'attributes': {
                        'actual_min': 1,
                        'actual_max': 2,
                    }
                },
                'x': {
                    'attributes': {
                        'actual_min': -2,
                        'actual_max': -1,
                    }
                },
            },
            'attributes': {
                'geospatial_lat_min': 1,
                'geospatial_lat_max': 2,
                'geospatial_lon_min': -2,
                'geospatial_lon_max': -1,
                'geospatial_bbox': 'POLYGON ((-1 1, -1 2, -2 2, -2 1, -1 1))',
                'geospatial_bounds': 'LINESTRING (-1 1, -2 2)',
                'geospatial_bounds_crs': 'EPSG:4326',
            }
        }

    def test_wrap_same_coords(self):

        geo = pd.DataFrame({
            'x': [-1, -1, -1],
            'y': [1, 1, 1]
        })

        meta = utils.get_geographic_attributes(geo)

        assert meta == {
            'variables': {
                'y': {
                    'attributes': {
                        'actual_min': 1,
                        'actual_max': 1,
                    }
                },
                'x': {
                    'attributes': {
                        'actual_min': -1,
                        'actual_max': -1,
                    }
                },
            },
            'attributes': {
                'geospatial_lat_min': 1,
                'geospatial_lat_max': 1,
                'geospatial_lon_min': -1,
                'geospatial_lon_max': -1,
                'geospatial_bbox': 'POLYGON ((-1 1, -1 1, -1 1, -1 1))',
                'geospatial_bounds': 'POINT (-1 1)',
                'geospatial_bounds_crs': 'EPSG:4326',
            }
        }
